@@ -108,7 +108,7 @@ Style::Layer::Filter::Filter(const QJsonArray &json)
 		_kv = QPair<QByteArray, QVariant>(json.at(1).toString().toUtf8(),
 		  QVariant());
 		for (int i = 2; i < json.size(); i++)
-			_set.insert(json.at(i).toString().toUtf8());
+			_set.append(variant(json.at(i)));
 	}  else if (type == "!in") {
 		if (json.size() < 3)
 			INVALID_FILTER(json);
@@ -117,7 +117,7 @@ Style::Layer::Filter::Filter(const QJsonArray &json)
 		_kv = QPair<QByteArray, QVariant>(json.at(1).toString().toUtf8(),
 		  QVariant());
 		for (int i = 2; i < json.size(); i++)
-			_set.insert(json.at(i).toString().toUtf8());
+			_set.append(variant(json.at(i)));
 	} else if (type == "has") {
 		if (json.size() < 2)
 			INVALID_FILTER(json);
@@ -200,7 +200,7 @@ bool Style::Layer::Filter::match(const PBF::Feature &feature) const
 			if (!(v = feature.value(_kv.first)))
 				return _not;
 			else
-				return _set.contains((*v).toByteArray()) ^ _not;
+				return _set.contains(*v) ^ _not;
 		case Has:
 			return (feature.value(_kv.first) ? true : false) ^ _not;
 		case All:
@@ -390,7 +390,7 @@ Style::Layer::Layout::Layout(const QJsonObject &json)
 
 	// icon
 	_icon = Template(FunctionS(json["icon-image"]));
-	_iconSize = FunctionF(json["icon-size"]);
+	_iconSize = FunctionF(json["icon-size"], 1.0);
 
 	// symbol
 	_symbolPlacement = FunctionS(json["symbol-placement"]);
@@ -415,9 +415,10 @@ Text::Anchor Style::Layer::Layout::textAnchor(int zoom) const
 {
 	QString anchor(_textAnchor.value(zoom));
 
-	if (anchor == "left")
+	if (anchor == "left" || anchor == "top-left" || anchor == "bottom-left")
 		return Text::Left;
-	else if (anchor == "right")
+	else if (anchor == "right" || anchor == "top-right"
+	  || anchor == "bottom-right")
 		return Text::Right;
 	else if (anchor == "top")
 		return Text::Top;
@@ -527,7 +528,7 @@ Style::Layer::Layer(const QJsonObject &json)
 
 bool Style::Layer::match(int zoom, const PBF::Feature &feature) const
 {
-	if (zoom >= 0 && (zoom < _minZoom || zoom > _maxZoom))
+	if (zoom >= 0 && (zoom < _minZoom || zoom >= _maxZoom))
 		return false;
 
 	return _filter.match(feature);
@@ -598,24 +599,27 @@ static bool loadSprites(const QDir &styleDir, const QString &json,
 	return true;
 }
 
-bool Style::load(const QString &fileName)
+Style::Style(const QString &fileName) : _valid(false)
 {
 	QFile file(fileName);
+	if (!file.exists())
+		return;
 	if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
-		qCritical() << fileName << ": error opening file";
-		return false;
+		qCritical() << fileName << ":" << file.errorString();
+		return;
 	}
 	QByteArray ba(file.readAll());
-	file.close();
 
 	QJsonParseError error;
 	QJsonDocument doc(QJsonDocument::fromJson(ba, &error));
 	if (doc.isNull()) {
 		qCritical() << fileName << ":" << error.errorString();
-		return false;
+		return;
 	}
-
 	QJsonObject json(doc.object());
+
+	_name = json["name"].toString();
+
 	if (json.contains("layers") && json["layers"].isArray()) {
 		QJsonArray layers = json["layers"].toArray();
 		for (int i = 0; i < layers.size(); i++)
@@ -627,7 +631,7 @@ bool Style::load(const QString &fileName)
 	loadSprites(styleDir, "sprite.json", "sprite.png", _sprites);
 	loadSprites(styleDir, "sprite@2x.json", "sprite@2x.png", _sprites2x);
 
-	return true;
+	_valid = true;
 }
 
 Sprites &Style::sprites(const QPointF &scale)
@@ -719,3 +723,14 @@ void Style::render(const PBF &data, Tile &tile)
 	//tile.painter().setRenderHint(QPainter::Antialiasing, false);
 	//tile.painter().drawRect(rect);
 }
+
+QStringList Style::layers() const
+{
+	QSet<QString> set;
+
+	for (int i = 0; i < _layers.size(); i++)
+		if (!_layers.at(i).sourceLayer().isEmpty())
+			set.insert(_layers.at(i).sourceLayer());
+
+	return set.values();
+}
